# 2018-10-15 David Goncalves <david@lestat.st>
#            Python class to communicate with a MAX17040 chip on I2C bus.
#            It's a 1-Cell Fuel Gauge with ModelGauge from Maxim Integrated.
#            > Code ported to MicroPython

import ustruct
import sys
from micropython import const


class umax17040:

    # Register addresses
    REG_VCELL = const(0x2)
    REG_SOC = const(0x4)
    REG_MODE = const(0x6)
    REG_VERSION = const(0x8)
    REG_RCOMP = const(0xc)
    REG_COMMAND = const(0xfe)

    # Values for mode register
    MODE_QSTRT = const(0x4000)

    # Values for command register
    CMD_POR = const(0x54)

    def __init__(self, bus, address=0x36):
        self.i2c_addr = address
        self.i2c_bus = bus

    def _write_data(self, address, data):
        self.i2c_bus.writeto_mem(self.i2c_addr, address, bytearray(data))

    def _read_data(self, address, size):
        return(self.i2c_bus.readfrom_mem(self.i2c_addr, address, size))

    def GetFirmwareVersion(self):
        return(ustruct.unpack('>H', self._read_data(self.REG_VERSION, 2))[0])

    def GetCellVoltage(self):
        return(int((ustruct.unpack('>H', self._read_data(self.REG_VCELL, 2))[0] >> 4) * 1.25))

    def GetStateOfCharge(self):
        data = self._read_data(self.REG_SOC, 2)
        soc = round(data[0] + (data[1] * (1/256.0)), 2)
        return(100.0 if soc > 100 else soc)
